unit Cybmaths;

interface

uses
  SysUtils, Windows, Graphics, dialogs{, cybsort};
  
type
    tfloat = double;

const
   maxdims = 4;
   worlddims = 3;
   scale_factor = 1000;
   max_block_size = 65520;
   max_element = 80;
{   trunc(sqrt(max_block_size / floatsize));}
   max_legend_items = 20;
   testdims = 11;
   no_pts = 10;
   floatsize = sizeof(tfloat);
   base = 1;
   two = 2;

type
    t2x1= array[base..two] of tfloat;
    t4x4 = array[base..maxdims, base..maxdims] of tfloat;
{    tmult = array[1..2] of t4x4;}
    t4x4_ptr = ^t4x4;
    t4x1 = array[base..maxdims] of tfloat;
    t3x1 = array[base.. worlddims] of tfloat;
    p4x1 = ^t4x1;
    pts = array[base..no_pts] of t4x1;
    scrpts = array[base..no_pts] of tpoint;

    ttest_axis_arr = array[base..testdims] of tfloat;
    ttest_surf_arr = array[base..testdims, base..testdims] of tfloat;
    tbytes = array[base..max_element*max_element*floatsize] of byte;
    taxis_arr = array[base..max_element] of tfloat;
    tsurf_arr = array[base..max_element*max_element] of tfloat;
(*    tdraw_arr = array[base..max_element*max_element] of tpoint;*)
    tcol_arr = array[base..max_element*max_element] of tpattern;
    paxis_arr = ^taxis_arr;
    psurf_arr = ^tsurf_arr;
{    pdraw_arr = ^tdraw_arr;}
    pcol_arr = ^tcol_arr;

    tsort_rec = record
        val : tfloat;
        indx : byte;
    end;

function tan(x : tfloat) : tfloat;
function ArcSin(x : tfloat) : tfloat;
function ArcCos(x : tfloat) : tfloat;
procedure mult_4x1(var m1p : t4x4; var m2p, resp : t4x1);
procedure mult_4x4(var m1p, m2p, resp : t4x4);
procedure translate(var mat : t4x4; trans : t4x1);
procedure rotate_x(var mat : t4x4; angle : tfloat);
procedure rotate_y(var mat : t4x4; angle : tfloat);
procedure rotate_z(var mat : t4x4; angle : tfloat);
procedure scale(var mat : t4x4; scaleby : t4x1);
function degtorad(angle : tfloat) : tfloat;
function radtodeg(angle : tfloat) : tfloat;

function t3x1_to_t4x1(inpm : t3x1) : t4x1;
function t4x1_to_t3x1(inpm : t4x1) : t3x1;
procedure draw_rotated_text(     hdc_hndl : HDC;
                                 angle : tfloat;
                                 posn : tpoint;
                                 msg : string);
{procedure sort_array( var x_arr, y_arr, ans_x, ans_y : paxis_arr;
                      var z_arr, ans_z : psurf_arr;
                      var mins, maxs : t3x1;
                      xdim, ydim : integer);}
function average(v1, v2 : tfloat) : tfloat;
function mid_span(v1, v2 : tfloat) : tfloat;
function lesser(t1,t2 : tfloat) : tfloat;
function greater(t1,t2 : tfloat) : tfloat;
function bool_getm(var P : pointer; Size: Word) : boolean;
function z_coord(xc, yc : byte) : integer;
function lessint(t1,t2 : longint) : longint;
function greatint(t1,t2 : longint) : longint;


const
   tx : ttest_axis_arr = (-3, 2, 0, 4, -5, -2, 3, -4, -1, 5, 1);
   ty : ttest_axis_arr = (1, -3, 0, 4, -5, 3, -2, -4, 2, 5, -1);
   tz :  ttest_surf_arr = ((4.12, 4.69, 5.1, 3.16, 1, 4.69, 4.12, 3.16, 5, 1, 5),
                         (5, 5.48, 5.83, 4.24, 3, 5.48, 5, 4.24, 5.74, 3, 5.74),
                         (4, 4.58, 5, 3, 0, 4.58, 4, 3, 4.9, 0, 4.9),
                         (5.66, 6.08, 6.4, 5, 4, 6.08, 5.66, 5, 6.32, 4, 6.32),
                         (6.4, 6.78, 7.07, 5.83, 5, 6.78, 6.4, 5.83, 7, 5, 7),
                         (5, 5.48, 5.83, 4.24, 3, 5.48, 5, 4.24, 5.74, 3, 5.74),
                         (4.47, 5, 5.39, 3.61, 2, 5, 4.47, 3.61, 5.29, 2, 5.29),
                         (5.66, 6.08, 6.4, 5, 4, 6.08, 5.66, 5, 6.32, 4, 6.32),
                         (4.47, 5, 5.39, 3.61, 2, 5, 4.47, 3.61, 5.29, 2, 5.29),
                         (6.4, 6.78, 7.07, 5.83, 5, 6.78, 6.4, 5.83, 7, 5, 7),
                         (4.12, 4.69, 5.1, 3.16, 1, 4.69, 4.12, 3.16, 5, 1, 5));
view_point_mults : t3x1 = (-0.25, -0.25, -2.25);
view_orig_mult : t3x1 = (-1, -1, -1);

implementation

var
     ax_arr  : paxis_arr;
     answer_x, answer_y : paxis_arr;
     tmp_z_arr, z_a, answer_z : psurf_arr;
     idim,xd, yd : byte;
     min_z, max_z  : tfloat;

function z_coord(xc, yc : byte) : integer;

begin
  result := ((yc-base)*xd) + xc;
end;

function bool_getm(var P : pointer; Size: Word) : boolean;
begin
  getmem(p,size);
  result := (p <> nil);
end;

function greater(t1,t2 : tfloat) : tfloat;

begin
    if (t1 < t2) then greater := t2
                 else greater := t1;
end;
function lesser(t1,t2 : tfloat) : tfloat;
begin
    if (t1 > t2) then result := t2
                 else result := t1;
end;

function lessint(t1,t2 : longint) : longint;
begin
    if (t1 > t2) then result := t2
                 else result := t1;
end;
function greatint(t1,t2 : longint) : longint;
begin
    if (t1 < t2) then greatint := t2
                 else greatint := t1;
end;

function average(v1, v2 : tfloat) : tfloat;
begin
     result := (v1 + v2)/2;
end;

function mid_span(v1, v2 : tfloat) : tfloat;
begin
     result := (v1 - v2)/2;
end;

function tan(x : tfloat) : tfloat;
var
 cx : tfloat;
begin
   cx := cos(x);
   if (cx <> 0) then  result := Sin(x) / Cx
                else  result := 0.0;
end;

function ArcSin(x : tfloat) : tfloat;
begin
     if (x <> 1) then result := ArcTan(x/sqrt (1-sqr(x)))
                 else result := 90;
end;

function ArcCos(x : tfloat) : tfloat;
begin
 if (x <> 0) then result := ArcTan (sqrt (1-sqr (x)) /x)
             else result := 0;
end;

procedure mult_4x1(var m1p : t4x4; var m2p, resp : t4x1);
var
  colcnt,rowcnt : byte;

begin
   fillchar(resp, sizeof(t4x1),0);
   for rowcnt := base to maxdims do begin
      for colcnt := base to maxdims do begin
         resp[rowcnt] := resp[rowcnt] + (m1p[rowcnt,colcnt]*m2p[colcnt]);
      end;
   end;
end;

procedure mult_4x4(var m1p, m2p, resp : t4x4);
var
  colcnt,rowcnt,extcnt : byte;

begin
   fillchar(resp, sizeof(t4x4),0);
   for rowcnt := base to maxdims do begin
     for colcnt := base to maxdims do begin
       for extcnt := base to maxdims do begin
          resp[rowcnt,colcnt] := resp[rowcnt,colcnt] + (m1p[rowcnt,extcnt]*m2p[extcnt,colcnt]);
       end;
     end;
   end;
end;

procedure translate(var mat : t4x4; trans : t4x1);
var
lc : byte;

begin
 fillchar(mat,sizeof(t4x4),0);
 for lc := base to maxdims do mat[lc,lc] := 1.0;
 mat[1,4] := trans[1];
 mat[2,4] := trans[2];
 mat[3,4] := trans[3];
end;

procedure rotate_x(var mat : t4x4; angle : tfloat);

var
  sm,cm : tfloat;

begin
 sm := sin(angle);
 cm := cos(angle);
 fillchar(mat,sizeof(t4x4),0);
 mat[1,1] := 1;
 mat[2,2] := cm;
 mat[2,3] := sm;
 mat[3,2] := -1*sm;
 mat[3,3] := cm;
 mat[4,4] := 1;
end;

procedure rotate_y(var mat : t4x4; angle : tfloat);

var
  sm,cm : tfloat;

begin
 sm := sin(angle);
 cm := cos(angle);
 fillchar(mat,sizeof(t4x4),0);
 mat[1,1] := cm;
 mat[1,3] := sm;
 mat[2,2] := 1;
 mat[3,1] := -1*sm;
 mat[3,3] := cm;
 mat[4,4] := 1;
end;

procedure rotate_z(var mat : t4x4; angle : tfloat);

var
  sm,cm : tfloat;

begin
 sm := sin(angle);
 cm := cos(angle);
 fillchar(mat,sizeof(t4x4),0);
 mat[1,1] := cm;
 mat[1,2] := -1*sm;
 mat[2,1] := sm;
 mat[2,2] := cm;
 mat[3,3] := 1;
 mat[4,4] := 1;
end;

procedure scale(var mat : t4x4; scaleby : t4x1);
var
lc : byte;

begin
 fillchar(mat,sizeof(t4x4),0);
 for lc := base to maxdims do mat[lc,lc] := scaleby[lc];
end;

function degtorad(angle : tfloat) : tfloat;

begin
   result := pi * angle / 180;
end;

function radtodeg(angle : tfloat) : tfloat;

begin
   result := 180 * angle / pi;
end;


function t3x1_to_t4x1(inpm : t3x1) : t4x1;
var
  op : t4x1;
  lc : byte;

begin
  for lc := base to worlddims do op[lc] := inpm[lc];
  op[maxdims] := 1;
  result := op;
end;

function t4x1_to_t3x1(inpm : t4x1) : t3x1;
var
  op : t3x1;
  lc : byte;

begin
  for lc := base to worlddims do op[lc] := inpm[lc];
  result := op;
end;

procedure draw_rotated_text(     hdc_hndl : HDC;
                                 angle : tfloat;
                                 posn : tpoint;
                                 msg : string);
var
(*  hdc_hndl : word;*)
  E : longint;
  font_struc_size, font_res, font_mem_size : word;
  pc,ls,lc : byte;
  glyph : TGlyphMetrics;
  M11: TFixed;
  M12: TFixed;
  M21: TFixed;
  M22: TFixed;
  rot_m : tmat2;
  ta, ca, sa : tfloat;
  bmp_size, cfrac, sfrac : word;
  cval, sval : integer;
  sparedc : hdc;
  sparebmp, resbmp, oldbmp : hbitmap;
  res_bmp : pointer;
  font_data : TOutlineTextMetric;
  font_dump : pointer;
{  ^TOutlineTextMetric;}
  x_p, y_p, w_p, h_p : integer;

  procedure set_rotm;
  begin
    font_mem_size :=  font_struc_size;
    getmem(font_dump, font_mem_size);
    font_res := GetOutlineTextMetrics(HDC_hndl,  font_struc_size, font_dump);
{   move(font_dump^ , font_data, font_mem_size);}
    with TOutlineTextMetric(font_dump^) do begin
        w_p :=  otmTextMetrics.tmMaxCharWidth;
        h_p := otmTextMetrics.tmHeight;
    end;
    freemem(font_dump, font_mem_size);
    ca := cos(angle);
    cfrac := round(frac(ca));
    cval := trunc(ca);
    sa := sin(angle);
    sfrac := round(frac(sa));
    sval := trunc(sa);
    if (ca <>0) then ta := sa/ca
                else ta := 1;
    WITH M11 do begin
        fract := cfrac;
        value := cval;
    end;
    m22 := m11;
    WITH M12 do begin
        fract := sfrac;
        value := -1*sval;
    end;
    WITH M21 do begin
        fract := sfrac;
        value := sval;
    end;
    with rot_m do begin
      eM11 := m11;
      eM12 := m12;
      eM21 := m21;
      eM22 := m22;
    end;
    ls := length(msg);
    x_p := posn.x;
    y_p := posn.y;
  end;

begin
  font_struc_size := GetOutlineTextMetrics(HDC_hndl, 0, nil);
  if (font_struc_size <> 0) then begin {a truetype font !}
   set_rotm;
   try
     sparedc := createcompatibleDC(hdc_hndl);
     E := 0;
     bmp_size := GetGlyphOutline(HDC_hndl, word('M'), GGO_BITMAP, Glyph, E, nil, rot_M);
     if (bmp_size <> -1)  then begin
        getmem(res_bmp, bmp_size);
        for lc := 1 to ls do begin
           E := bmp_size;
           font_res := GetGlyphOutline(HDC_hndl, word(msg[lc]), GGO_BITMAP, Glyph, E, res_bmp, rot_M);
           resbmp := CreateBitmap(W_p, H_p , Planes, bmp_size, res_bmp);
           if (lc = 1) then oldbmp := selectobject(sparedc, resbmp)
                       else sparebmp := selectobject(sparedc, resbmp);
           pc := pred(lc);
           x_p := x_p + (pc*w_p);
           y_p := y_p + round(pc*h_p*ta);
           bitblt(hdc_hndl,x_p,y_p,w_p,h_p,sparedc,0,0,srcCopy);
        end;
        freemem(res_bmp, bmp_size);
     end;
   finally
         selectobject(sparedc, oldbmp);
         deletedc(sparedc);
   end;
  end
  else begin
     MessageDlg('Non True Type Font. Cannot be rotated (easily !)', mterror, [mbOK] , 0);
  end;
end;
{$F+}
(*procedure sort_in;
  var
    lc : byte;
    in_sort_rec : tsort_rec;

  begin
     for lc := base to idim do begin
       with in_sort_rec do begin
         val := ax_arr^[lc];
         indx := lc;
       end;
       sortrelease(in_sort_rec);
     end;
  end;

  function sort_less( var a, b) : boolean;
   begin
     result := (tsort_rec(a).val < tsort_rec(b).val);
   end;

  procedure x_sort_out;
  var
    nd,lc2, lc : byte;
    out_sort_rec : tsort_rec;
    tmp : tfloat;
    fr : boolean;

  begin
    lc := 1;
    fr := true;
    repeat
       sortreturn(out_sort_rec);
       answer_x^[lc] := out_sort_rec.val;
       nd := out_sort_rec.indx;
       for lc2 := base to yd do begin
              tmp := z_a^[z_coord(nd,lc2)];
              answer_z^[z_coord(lc,lc2)] :=  tmp;
              if fr then begin
                fr := false;
                min_z := tmp;
                max_z := tmp;
              end;
              max_z := greater(max_z ,tmp);
              min_z := lesser(min_z, tmp);
       end;
       inc(lc);
    until (sorteos or (lc > xd));
  end;

  procedure y_sort_out;
  var
    nd, lc2, lc : byte;
    out_sort_rec : tsort_rec;

  begin
    lc := 1;
    repeat
       sortreturn(out_sort_rec);
       answer_y^[lc] := out_sort_rec.val;
       nd := out_sort_rec.indx;
       for lc2 := base to xd do begin
              tmp_z_arr^[z_coord(lc2,lc)] :=  z_a^[z_coord(lc2,nd)];
       end;
       inc(lc);
    until (sorteos or (lc > yd));
  end;


procedure sort_array( var x_arr, y_arr, ans_x, ans_y : paxis_arr;
                      var z_arr, ans_z : psurf_arr;
                      var mins, maxs : t3x1;
                      xdim, ydim : integer);

var
  bufsize, sortresult : longint;
  ip, ep : proc;
  l : less_bool;

begin
    bufsize := xdim * ydim * floatsize;
    getmem(tmp_z_arr, bufsize);
    ax_arr := x_arr;
    answer_x := ans_x;
    answer_y := ans_y;
    answer_z := ans_z;
    z_a := z_arr;
    xd := xdim;
    yd := ydim;
    idim := xd;
    ip := sort_in;
    ep := x_sort_out;
    l := sort_less;
    sortresult := turbosort(sizeof(tsort_rec), ip, ep, l);
    ax_arr := y_arr;
    ep := y_sort_out;
    idim := ydim;
    z_a := answer_z;
    sortresult := turbosort(sizeof(tsort_rec), ip, ep, l);
    mins[1] := ans_x^[base];
    mins[2] := ans_y^[base];
    mins[3] := min_z;
    maxs[1] := ans_x^[xdim];
    maxs[2] := ans_y^[ydim];
    maxs[3] := max_z;
{    ans_z := tmp_z_arr;}
    z_a := ans_z;
    ans_z := tmp_z_arr;
    tmp_z_arr := z_a;
    freemem(tmp_z_arr, bufsize);
end;
*)
end.

(*
   tx : ttest_axis_arr =
  {  1   2    3     4     5    6     7     8     9          }
  ( 3.0, 5.3, 6.8, -12.5, 5.7, 1.25, 0.67, 112 , 56, 0.003);
  ty : ttest_axis_arr =
  ( 2.0, 11.3, 4.8, -1.5, -5.7, 10.25, 20.67, 1120 , 1056, -100.003);
  tz :  ttest_surf_arr =
   (( 32.0, 311.3, 44.8, -41.5, -55.7, 110.25, 120.67, 120 , 1156, 1500.003),
{2} ( 22.0, 411.3, 24.8, 21.5, 3055.7, 4110.25, 5420.67, 1120 , 1056, 110.003),
{3} ( 232.0, 771.3, 354.8, 21.5, 4.7, 0.25, 0.67, 110 , 156, 40.003),
{4} ( 4.0, 161.3, -54.8, -31.5, -25.7, 110.25, 120.67, 120 , 056, 310.003),
{5} ( 112.0, -211.3, 34.8, 145.5, 555.7, -51.25, -2005.67, 120 , 56, 1103),
{6} ( 52.0, -11.3, 44.8, -561.5, -325.7, 550.25, 720.67, 1120 , 3056, 0.003),
{7} ( -325.0, 511.13, 14.8, -11.5, -15.7, 410.25, 420.67, 620 , 1534, 100.003),
{8} ( 2.0, 11.3, 4.8, -1.5, -5.7, 10.25, 20.67, 1120 , 1056, 100.003),
{9} ( 2.0, 11.3, 4.8, -1.5, -5.7, 10.25, 20.67, 1120 , 1056, 100.003),
{10} ( 2.0, 11.3, 4.8, -1.5, -5.7, 10.25, 20.67, 1120 , 1056, 100.003));

*)
